/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.providers;

import java.io.Externalizable;

import javax.xml.namespace.QName;

import org.apache.wsif.logging.Trc;

/**
 * This class encapsultes a simple association between XML and Java type
 * (QName and Class).
 *
 * @author Alekander Slominski
 * @author Owen Burroughs <owenb@apache.org>
 */
public class WSIFDynamicTypeMapping implements Externalizable {
	private static final long serialVersionUID = 1L;
    protected QName xmlType;
    protected Class javaType;

    /**
     * No args constructor required for deserialization purposes.
     * This constructor should not be used directly.
     */
    public WSIFDynamicTypeMapping() {
    	xmlType = null;
    	javaType = null;
    }

    public WSIFDynamicTypeMapping(QName xmlType, Class javaType) {
        Trc.entry(this, xmlType, javaType);
        this.xmlType = xmlType;
        this.javaType = javaType;
        Trc.exit();
    }

    public QName getXmlType() {
        return xmlType;
    }

    public Class getJavaType() {
        return javaType;
    }

    public String toString() {
        return "QName:" + xmlType + " Class:" + javaType;
    }

    /**
     *  Override writeExternal method to allow primitive classes to be serialized. 
     */
    public void writeExternal(java.io.ObjectOutput out)
        throws java.io.IOException {
        out.writeObject(xmlType);
        if (javaType.isPrimitive()) {
            out.writeBoolean(true);
            out.writeObject(javaType.getName());
        } else {
            out.writeBoolean(false);
            out.writeObject(javaType);
        }

    }
    
    /**
     *  Override readExternal method to allow primitive classes to be deserialized. 
     */
    public void readExternal(java.io.ObjectInput in)
        throws java.io.IOException, ClassNotFoundException {
        xmlType = (QName) in.readObject();
        boolean primitive = in.readBoolean();
        if (primitive) {
            String primitiveClassName = (String) in.readObject();
            if (primitiveClassName.equals("int"))
                javaType = int.class;
            else if (primitiveClassName.equals("float"))
                javaType = float.class;
            else if (primitiveClassName.equals("double"))
                javaType = double.class;
            else if (primitiveClassName.equals("boolean"))
                javaType = boolean.class;
            else if (primitiveClassName.equals("long"))
                javaType = long.class;
            else if (primitiveClassName.equals("short"))
                javaType = short.class;
            else if (primitiveClassName.equals("byte"))
                javaType = byte.class;
            else if (primitiveClassName.equals("void"))
                javaType = void.class;
        } else
            javaType = (Class) in.readObject();
    }
}