/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.wsif.providers.jca.toolplugin;

import javax.wsdl.*;

/**
 * The <code>PortTypeSelection</code> class is the representation of a selected PortType
 * that is passed to the <code>getDefinition</code> operation of the ImportService. Tool environments
 * are responsible for constructing instances of this object and passing them to the Import service.
 *  @author Hesham Fahmy <hfahmy@ca.ibm.com>
 *  @author Piotr Przybylski <piotrp@ca.ibm.com>
 */
public class PortTypeSelection implements java.io.Serializable {

	private static final long serialVersionUID = 1L;
	private javax.xml.namespace.QName fieldPortTypeQName = null;
	private OperationSelection[] fieldOperationSelection = null;

	/**
	 * Constructs a <code>PortTypeSelection</code> for a given WSDL PortType and selected WSDL Operations
	 * @param <code>portTypeQName</code> The QName of the WSDL PortType selected
	 * @param <code>operationSelection</code> The set of WSDL operations that are selected for the given WSDL PortType.
	 */
	public PortTypeSelection(javax.xml.namespace.QName portTypeQName, OperationSelection[] operationSelection) {
		
		this.fieldPortTypeQName = portTypeQName;
		this.fieldOperationSelection = operationSelection;
	}

	/**
	 * Returns the QName of the selected WSDL PortType.
	 * @return QName
	 */
	public javax.xml.namespace.QName getPortTypeQName() {
		return this.fieldPortTypeQName;
	}
	
	/**
	 * Sets the QName of the selected WSDL PortType.
	 * @param <code>portTypeQName</code> The QName to set
	 */
	public void setPortTypeQName(javax.xml.namespace.QName portTypeQName) {
		this.fieldPortTypeQName = portTypeQName;
	}


	/**
	 * Returns the selected WSDL Operations for the PortType. The selected operations are represented by an array of <code>OperationSelection</code>
	 * objects.
	 * @return Returns a OperationSelection[]
	 */
	public OperationSelection[] getOperationSelection() {
		return fieldOperationSelection;
	}

	/**
	 * Sets the selected WSDL Operations for the PortType.
	 * @param <code>operationSelection</code> The operationSelection to set.
	 */
	public void setOperationSelection(OperationSelection[] operationSelection) {
		fieldOperationSelection = operationSelection;
	}

}