/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.providers.soap.apacheaxis;

import java.util.ArrayList;
import java.util.Arrays;

/**
 * Various constants used by the WSIF AXIS provider classes
 * 
 * @author Ant Elder <ant.elder@uk.ibm.com>
 */
public class WSIFAXISConstants {

	public static final String SOAP_BINDING_NAMESPACE =
		"http://schemas.xmlsoap.org/wsdl/soap/";

	public static final String HTTP_TRANSPORT_URI =
		"http://schemas.xmlsoap.org/soap/http";

	public static final String JMS_TRANSPORT_URI =
		"http://schemas.xmlsoap.org/soap/jms";

	public static final ArrayList VALID_TRANSPORTS =
		new ArrayList(
			Arrays.asList(
				new String[] { HTTP_TRANSPORT_URI, JMS_TRANSPORT_URI }));

	public static final String CLASS_IN_AXIS_JAR = "org.apache.axis.AxisEngine";

	public static final String CLASS_IN_JMS_JAR = "javax.jms.Queue";

	public static final String DEFAULT_SOAP_ENCODING_URI =
		"http://schemas.xmlsoap.org/soap/encoding/";

	public static final String STYLE_RPC = "rpc";
	public static final String STYLE_DOCUMENT = "document";
	public static final ArrayList VALID_STYLES =
		new ArrayList(
			Arrays.asList(new String[] { STYLE_RPC, STYLE_DOCUMENT }));

	public static final String USE_ENCODED = "encoded";
	public static final String USE_LITERAL = "literal";
	public static final ArrayList VALID_USES =
		new ArrayList(
			Arrays.asList(new String[] { USE_ENCODED, USE_LITERAL }));

	public static final String AXIS_STYLE_MESSAGE = "message";
	public static final String AXIS_STYLE_WRAPPED = "wrapped";

    /**
     *  WSIF context part name for any default type serializers
     *  The context value should be an ArrayList of TypeSerializer objects  
     */
    public static final String CONTEXT_DEFAULT_SOAP_TYPE_SERIALIZERS =
        "org.apache.wsif.axis.default.type.serializers";

    /**
     *  WSIF context part name to override default (de)serializers for a type  
     *  The context value should be an ArrayList of TypeSerializer objects  
     */
    public static final String CONTEXT_SOAP_TYPE_SERIALIZERS =
        "org.apache.wsif.axis.type.serializers";

}
