/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.providers.soap.apacheaxis;

import org.apache.axis.AxisEngine;
import org.apache.axis.AxisFault;
import org.apache.axis.MessageContext;
import org.apache.axis.client.Call;
import org.apache.axis.client.Transport;
import org.apache.wsif.WSIFException;
import org.apache.wsif.WSIFOperation;
import org.apache.wsif.logging.Trc;
import org.apache.wsif.util.jms.WSIFJMSDestination;

/**
 * @author Mark Whitlock <whitlock@apache.org>
 * @author Ant Elder <ant.elder@uk.ibm.com>
 */
public class WSIFJmsTransport extends Transport {
    private WSIFJMSDestination destination = null;
    private String asyncOperation = "false";
    private WSIFOperation wsifOperation = null;
    private Long syncTimeout = null;
    private Long asyncTimeout = null;

    public static final String DESTINATION = "destination";
    public static final String ASYNCOPERATION = "asyncOperation";
    public static final String WSIFOPERATION = "wsifOperation";
    public static final String SYNC_TIMEOUT = "syncTimeout";
    public static final String ASYNC_TIMEOUT = "asyncTimeout";

    public WSIFJmsTransport(WSIFJMSDestination destination) throws WSIFException {
    	if (destination == null) {
    		throw new WSIFException("destination is null");
    	}
    	this.destination = destination;
    }

    public void setDestination(WSIFJMSDestination destination) {
        Trc.entry(this, destination);
        this.destination = destination;
        Trc.exit();
    }

    public void setAsyncOperation(String asyncOperation) {
        Trc.entry(this, asyncOperation);
        this.asyncOperation = asyncOperation;
        Trc.exit();
    }

    public void setWsifOperation(WSIFOperation wsifOperation) {
        Trc.entry(this, wsifOperation);
        this.wsifOperation = wsifOperation;
        Trc.exit();
    }

    public void setSyncTimeout(Long syncTimeout) {
        Trc.entry(this, syncTimeout);
        this.syncTimeout = syncTimeout;
        Trc.exit();
    }

    public void setAsyncTimeout(Long asyncTimeout) {
        Trc.entry(this, asyncTimeout);
        this.asyncTimeout = asyncTimeout;
        Trc.exit();
    }

    public WSIFJMSDestination getDestination() {
        Trc.entry(this);
        Trc.exit(this.destination);
        return this.destination;
    }

    public String getAsyncOperation() {
        Trc.entry(this);
        Trc.exit(this.asyncOperation);
        return this.asyncOperation;
    }

    public WSIFOperation getWsifOperation() {
        Trc.entry(this);
        Trc.exit(this.wsifOperation);
        return this.wsifOperation;
    }

    public Long getSyncTimeout() {
        Trc.entry(this);
        Trc.exit(this.syncTimeout);
        return this.syncTimeout;
    }

    public Long getAsyncTimeout() {
        Trc.entry(this);
        Trc.exit(this.asyncTimeout);
        return this.asyncTimeout;
    }

    public void setupMessageContextImpl(
        MessageContext context,
        Call call,
        AxisEngine engine)
        throws AxisFault {
        Trc.entry(this, context, call, engine);
        context.setTransportName("jms");
        if (destination != null)
            context.setProperty(DESTINATION, destination);
        context.setProperty(ASYNCOPERATION, new Boolean(asyncOperation));
        if (wsifOperation != null)
            context.setProperty(WSIFOPERATION, wsifOperation);
        if (syncTimeout != null)
            context.setProperty(SYNC_TIMEOUT, syncTimeout);
        if (asyncTimeout != null)
            context.setProperty(ASYNC_TIMEOUT, asyncTimeout);
        Trc.exit();
    }

    public WSIFJmsTransport copy() throws WSIFException {
        Trc.entry(this);
        WSIFJmsTransport t = new WSIFJmsTransport(destination);
        t.setAsyncOperation(asyncOperation);
        t.setWsifOperation(wsifOperation);
        t.setSyncTimeout(syncTimeout);
        t.setAsyncTimeout(asyncTimeout);
        if (Trc.ON)
            Trc.exit(t.deep());
        return t;
    }

    public void close() throws WSIFException {
        Trc.entry(this);
    	if (destination == null) {
    		throw new WSIFException("already closed");
    	}
   		destination.close();
   		destination = null;
        Trc.exit();
    }
    
    public String deep() {
        String buff = "";
        try {
            buff = new String(super.toString() + ":\n");

            buff += "destination:" + destination;
            buff += "asyncOperation:" + asyncOperation;
            buff += "wsifOperation:" + wsifOperation;
            buff += "syncTimeout:" + syncTimeout;
            buff += "asyncTimeout:" + asyncTimeout;
        } catch (Exception e) {
            Trc.exceptionInTrace(e);
        }
        return buff;
    }
}