/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.schema;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Vector;

import javax.xml.namespace.QName;

import org.apache.wsif.WSIFConstants;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * A class to represent a &lt;complexType&gt; element in a schema
 * 
 * @author Owen Burroughs <owenb@apache.org>
 */
public class ComplexType extends SchemaType implements Serializable {

	static final long serialVersionUID = 1L;

    private boolean isAnArray = false;
    private String name = "";
    private QName typeName = null;
    private QName arrayType = null;
    private int arrayDim = 0;
    private ComplexContent complexContent = null;
    private static final QName soapEncArray =
        new QName(WSIFConstants.NS_URI_SOAP_ENC, "Array");
    private static final QName soapEncArrayType =
        new QName(WSIFConstants.NS_URI_SOAP_ENC, "arrayType");
    private static final QName wsdlArrayType =
        new QName(WSIFConstants.NS_URI_WSDL, "arrayType");
    ArrayList sequenceElements = new ArrayList();        

	/**
	 * Constructor
	 * @param el The dom element for this complexType
	 */
    ComplexType(Element el, String tns) {
        typeName = getAttributeQName(el, "name", tns);
        if (typeName != null) {
        	name = typeName.getLocalPart();
        }
        
        process(el, tns);
        
        if (name.startsWith("ArrayOf")) {
            if (complexContent != null) {
                Restriction res = complexContent.getRestriction();
                if (res != null) {
                    QName base = res.getBase();
                    if (soapEncArray.equals(base)) {
                        Attribute[] atts = res.getAttributes();
                        if (atts != null && atts.length > 0) {
                            for (int i = 0; i < atts.length; i++) {
                                Attribute a = atts[i];
                                if (a != null) {
                                    QName ref = a.getXMLAttribute("ref");
                                    if (soapEncArrayType.equals(ref)) {
                                        QName tempType =
                                            a.getXMLAttribute(wsdlArrayType);
                                        if (tempType != null) {
                                            String ns =
                                                tempType.getNamespaceURI();
                                            String lp = tempType.getLocalPart();
                                            // Work out array dimension
                                            int index = lp.lastIndexOf("[]");
                                            while(index != -1) {
                                            	lp = lp.substring(0, index);
                                            	arrayDim++;
                                            	index = lp.lastIndexOf("[]");
                                            }                                           
                                            arrayType = new QName(ns, lp);
                                        }
                                        break;
                                    }
                                }
                            }
                        } else {
                            SequenceElement[] sels = res.getSequenceElements();
                            if (sels != null && sels.length == 1) {
								SequenceElement sel = sels[0];
								QName tempType = sel.getXMLAttribute("type");
								if (tempType != null) {
									String ns = tempType.getNamespaceURI();
                                    String lp = tempType.getLocalPart();
                                    arrayType = new QName(ns, lp);
								}
                            }
                        }
                    }
                }
                isAnArray = true;
            }
        } else {
        }
    }

	/**
	 * @see SchemaType#isComplex()
	 */ 
    public boolean isComplex() {
        return true;
    }

	/**
	 * @see SchemaType#isArray()
	 */ 
    public boolean isArray() {
        return isAnArray;
    }

	/**
	 * @see SchemaType#getArrayType()
	 */ 
    public QName getArrayType() {
        return arrayType;
    }

	/**
	 * @see SchemaType#getArrayDimension()
	 */ 
    public int getArrayDimension() {
        return arrayDim;
    }

	/**
	 * @see SchemaType#getTypeName()
	 */ 
    public QName getTypeName() {
        return typeName;
    }

	/**
	 * Get all the &lt;element&gt; elements within a sequence nested in this complexType
	 * @return The &lt;element&gt; elements within the sequnce
	 */
    public SequenceElement[] getSequenceElements() {
        return (SequenceElement[]) sequenceElements.toArray(
            new SequenceElement[sequenceElements.size()]);
    }

    private void process(Element el, String tns) {
        NodeList children = el.getChildNodes();
        for (int i = 0; i < children.getLength(); i++) {
            Node child = children.item(i);
            if (child.getNodeType() == Node.ELEMENT_NODE) {
                Element subEl = (Element) child;
                String elType = subEl.getLocalName();
                if (elType.equals("complexContent")) {
                    complexContent = new ComplexContent(subEl, tns);
                } else if (elType.equals("sequence")) {
                    parseSequenceElements(subEl, tns);
                }
            }
        }
    }
    
    private void parseSequenceElements(Element el, String tns) {
        NodeList children = el.getChildNodes();
        for (int i = 0; i < children.getLength(); i++) {
            Node child = children.item(i);
            if (child.getNodeType() == Node.ELEMENT_NODE) {
                Element subEl = (Element) child;
                String elType = subEl.getLocalName();
                if (elType.equals("element")) {
                    sequenceElements.add(new SequenceElement(subEl, tns));
                }
            }
        }
    }    
}
