/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package simpletypes;

import java.math.BigDecimal;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.wsif.WSIFMessage;
import org.apache.wsif.WSIFOperation;
import org.apache.wsif.WSIFPort;
import org.apache.wsif.WSIFService;
import org.apache.wsif.WSIFServiceFactory;
import org.apache.wsif.providers.soap.apacheaxis.WSIFDynamicProvider_ApacheAxis;
import org.apache.wsif.util.WSIFPluggableProviders;
import util.TestUtilities;

/**
 * Junit test to test out using SOAP-ENC simple types
 * @author Owen Burroughs
 */
public class SOAPENCTest extends TestCase {
    String wsdlLocation =
        TestUtilities.getWsdlPath("java\\test\\simpletypes") + "SOAPENCTest.wsdl";
    static String server = TestUtilities.getSoapServer().toUpperCase();

    public SOAPENCTest(String name) {
        super(name);
    }

    public static void main(String[] args) {
//        TestUtilities.startListeners();
        junit.textui.TestRunner.run(suite());
//        TestUtilities.stopListeners();
    }

    public static Test suite() {
        return new TestSuite(SOAPENCTest.class);
    }

    public void setUp() {
        TestUtilities.setUpExtensionsAndProviders();
    }

    public void testAxis() {
        doit(server+"Port", "axis");
    }
    public void testSoap() {
        doit(server+"Port", "soap");
    }

    private void doit(String portName, String protocol) {
    	TestUtilities.setProviderForProtocol( protocol );

        try {
            WSIFServiceFactory factory = WSIFServiceFactory.newInstance();
            WSIFService service =
                factory.getService(wsdlLocation, null, // serviceNS
                null, // serviceName
                "http://wsifservice.simpletypes/", // portTypeNS
                "SimplePT"); // portTypeName

            WSIFPort port = service.getPort(portName);
			
			int i = 10;
			byte b = (byte) i;

            invokeOperation(port, "getString", "abcd");
            invokeOperation(port, "getBoolean", new Boolean(true));
            invokeOperation(port, "getFloat", new Float(4321));
            invokeOperation(port, "getDouble", new Double(4321));
            invokeOperation(port, "getDecimal", new BigDecimal(4321));
            invokeOperation(port, "getInteger", new Integer(4321));
            invokeOperation(port, "getShort", new Short("4321"));
            invokeOperation(port, "getByte", new Byte(b));
            invokeOperation(port, "getBase64", new byte[] {b});
        } catch (Exception e) {
            System.err.println(
                "SimpleTest(" + portName + ") caught exception " + e);
            e.printStackTrace();
            assertTrue(false);
        } finally {
        	TestUtilities.resetDefaultProviders();
        }
    }

    private void invokeOperation(WSIFPort port, String name, Object partVal)
        throws Exception {
        WSIFOperation operation = port.createOperation(name);

        WSIFMessage inputMessage = operation.createInputMessage();
        WSIFMessage outputMessage = operation.createOutputMessage();
        WSIFMessage faultMessage = operation.createFaultMessage();
        inputMessage.setObjectPart("dummy", partVal);

        boolean b =
            operation.executeRequestResponseOperation(
                inputMessage,
                outputMessage,
                faultMessage);

        if (b) {
            System.out.println(
                name + " returned: "
                    + outputMessage.getObjectPart("return"));
        } else {
            System.out.println(name + " failed!");
        }
    }
}
