/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package soap;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.wsif.WSIFMessage;
import org.apache.wsif.WSIFOperation;
import org.apache.wsif.WSIFPort;
import org.apache.wsif.WSIFService;
import org.apache.wsif.WSIFServiceFactory;
import org.apache.wsif.providers.soap.apacheaxis.WSIFDynamicProvider_ApacheAxis;
import org.apache.wsif.util.WSIFPluggableProviders;
import util.TestUtilities;

import addressbook.wsiftypes.Address;
import addressbook.wsiftypes.Phone;

/**
 * Junit test to test leaving out a part in the inpuyt message.
 * Missing parts should default to null.
 * @author Ant Elder <ant.elder@uk.ibm.com>
 */
public class MissingInputPartTest extends TestCase {
    String wsdlLocation =
        TestUtilities.getWsdlPath("java\\test\\addressbook\\wsifservice")
            + "AddressBook.wsdl";
    static String server = TestUtilities.getSoapServer().toUpperCase();

    public MissingInputPartTest(String name) {
        super(name);
    }

	public static void main(String[] args) {
        TestUtilities.startListeners(
            TestUtilities.ADDRESSBOOK_LISTENER
                | TestUtilities.NATIVEJMS_LISTENER);

	    junit.textui.TestRunner.run (suite());
        TestUtilities.stopListeners();	
	}

    public static Test suite() {
        return new TestSuite(MissingInputPartTest.class);
    }

    public void setUp() {
        TestUtilities.setUpExtensionsAndProviders();
    }

    public void testDynamicSOAP() {
        doitDyn(server+"Port", "soap");
    }
    public void testDynamicJava() {
        doitDyn("JavaPort", "java");
    }
    public void testDynamicSoapJms() {
        doitDyn("SOAPJMSPort", "soap");
    }
// TODO Axis doesn't work as we don't have an axis service
//     the soap service doesn't understand the axis nil value
//    public void testDynamicAxis() {
//        doitDyn(server+"Port", "axis");
//    }
//    public void testDynamicAxisJms() {
//        doitDyn("SOAPJMSPort", "axis");
//    }
    public void testDynamicNativeJms() {
        doitDyn("NativeJmsPort", "" ); 
    }

    private void doitDyn(String portName, String protocol) {

        if (portName.toUpperCase().indexOf("JMS") != -1 
        && !TestUtilities.areWeTesting("jms"))
            return;

        TestUtilities.setProviderForProtocol( protocol );

        try {
            WSIFServiceFactory factory = WSIFServiceFactory.newInstance();
                WSIFService service = factory.getService(wsdlLocation, null, // serviceNS 
                null, // serviceName 
                "http://wsifservice.addressbook/", // portTypeNS 
                "AddressBook"); // portTypeName 

            service.mapType(
                new javax.xml.namespace.QName(
                    "http://wsiftypes.addressbook/",
                    "address"),
                Class.forName("addressbook.wsiftypes.Address"));

            service.mapType(
                new javax.xml.namespace.QName(
                    "http://wsiftypes.addressbook/",
                    "phone"),
                Class.forName("addressbook.wsiftypes.Phone"));

            WSIFPort port = null;

            port = service.getPort(portName);

            WSIFOperation operation =
                port.createOperation("addEntry", "AddEntryWholeNameRequest", null);

            WSIFMessage inputMessage = operation.createInputMessage();
            WSIFMessage outputMessage = operation.createOutputMessage();
            WSIFMessage faultMessage = operation.createFaultMessage();

            // Create a name and address to add to the addressbook 
            String nameToAdd = "Chris P. Bacon";

            // Add the name and leave out address to the input message 
            inputMessage.setObjectPart("name", nameToAdd);
            //inputMessage.setObjectPart("address", null); // should default to null

            // Execute the operation, obtaining a flag to indicate its success 
            boolean ok =
                operation.executeRequestResponseOperation(
                    inputMessage,
                    outputMessage,
                    faultMessage);

            assertTrue( "Failed to add name and address to addressbook", ok );

            // Start from fresh 
            operation = null;
            inputMessage = null;
            outputMessage = null;
            faultMessage = null;

            operation = port.createOperation("getAddressFromName");

            // Create the messages 
            inputMessage = operation.createInputMessage();
            outputMessage = operation.createOutputMessage();
            faultMessage = operation.createFaultMessage();

            // Set the name to find in the addressbook 
            String nameToLookup = "Chris P. Bacon";
            inputMessage.setObjectPart("name", nameToLookup);

            // Execute the operation 
            ok = operation.executeRequestResponseOperation(
                    inputMessage,
                    outputMessage,
                    faultMessage);

            assertTrue( "Failed to lookup name from addressbook", ok );

            // Should have defaulted to null
            Address addressFound = (Address) outputMessage.getObjectPart("address");
            if ( addressFound != null ) {
               assertTrue( "city not null!!", addressFound.getCity() == null );
               assertTrue( "PhoneNumber not null!!", addressFound.getPhoneNumber() == null );
               assertTrue( "State not null!!", addressFound.getState() == null );
               assertTrue( "StreetName not null!!", addressFound.getStreetName() == null );
            }

        } catch (Exception ex) {
            ex.printStackTrace();
            assertTrue("AddressBookTest(" + portName + ") caught exception " + ex, false);
        } finally {
            if (protocol.equals("axis")) {
               WSIFPluggableProviders.overrideDefaultProvider(
                    "http://schemas.xmlsoap.org/wsdl/soap/",
                    null);
            }
        }
    }

}
